// Copyright (C) 2005, International Business Machines
// Corporation and others.  All Rights Reserved.
#if defined(_MSC_VER)
// Turn off compiler warning about long names
#  pragma warning(disable:4786)
#endif

#include <cassert>
#include <iomanip>
#include <string>
using namespace std;



// For Branch and bound
#include "CbcModel.hpp"
#include "CbcStrategy.hpp"
#include "OsiClpSolverInterface.hpp"
// Preprocessing
#include "CglPreProcess.hpp"

#include  "CoinTime.hpp"
#include "javacbc.hpp"
#include "CoinPackedMatrix.hpp"

CoinPackedMatrix CPM(int numberRows, int numberColumns, int numberElements, int *startint, int nbelems, int *length, int *rows, double *elements){
    if(nbelems>0){
        CoinBigIndex *start;
        start = new CoinBigIndex[nbelems];
        int i=0;
        for(i=0;i<nbelems;i++){
            start[i] = startint[i];
        }
        CoinPackedMatrix matrix(true,numberRows,numberColumns,numberElements,elements,rows,start,length);
        return matrix;
    }
    CoinPackedMatrix matrix;
    return matrix;
}

CoinPackedMatrix CPM(int *rowIndices, int *colIndices, double *elements, int nbelems){
    if(nbelems>0){
        CoinPackedMatrix matrix(true,rowIndices,colIndices,elements,nbelems);
        return matrix;
    }
    CoinPackedMatrix matrix;
    return matrix;
}

void iniSolve(OsiClpSolverInterface& solver){
  solver.initialSolve();
//  return solver;
}

void setInt(OsiClpSolverInterface& solver, int i){
  solver.setInteger(i);
//  return solver;
}

OsiClpSolverInterface newSolver(){
  OsiClpSolverInterface solver1;
  return solver1;
}

OsiClpSolverInterface makeModel(int numberRows, int numberColumns, int numberElements, int *startint, int nbelems, int *length, int *rows, double *elements, double *objective, double *rowLower, double *rowUpper, double *colLower, double *colUpper){
    OsiClpSolverInterface solver1;
    if(nbelems>0){
        CoinBigIndex *start;
        start = new CoinBigIndex[nbelems];
        int i=0;
        for(i=0;i<nbelems;i++){
            start[i] = startint[i];
        }
        CoinPackedMatrix matrix(true,numberRows,numberColumns,numberElements,elements,rows,start,length);
        // load problem
        solver1.loadProblem(matrix,colLower,colUpper,objective,rowLower,rowUpper);
    }
    return solver1;
}


CoinBigIndex CBI(int start[]){
  int length = sizeof(start)/sizeof(int);
  CoinBigIndex cbi[length];
  for(int i=0;i<length;i++){
    cbi[i] = start[i];
  }
  return *cbi;
}


int solve_model(OsiClpSolverInterface solver1, bool preProcess, bool docliques, int nbpasses, int minutes){
  double time1 = CoinCpuTime();

  // See if we want preprocessing
  OsiSolverInterface * solver2=&solver1;
  CglPreProcess process;
  /* docliques=false => Do not try and produce equality cliques */
  if (preProcess) {
    /* Do not try and produce equality cliques and
       do up to 5 passes */
    solver2 = process.preProcess(solver1,docliques,nbpasses);
    if (!solver2) {
      printf("Pre-processing says infeasible\n");
      exit(2);
    }
    solver2->resolve();
  }
  CbcModel model(*solver2);
  // If time is given then stop after that number of minutes
  if (minutes>=0.0) {
    std::cout<<"Stopping after "<<minutes<<" minutes"<<std::endl;
    model.setDblParam(CbcModel::CbcMaximumSeconds,60.0*minutes);
  }
  // Set strategy - below is == CbcStrategyDefault()
  CbcStrategyDefault strategy(true,5,0);
  model.setStrategy(strategy);
  // Do complete search
  
  model.branchAndBound();
  std::cout<<"Problem took "<<CoinCpuTime()-time1<<" seconds, "
	   <<model.getNodeCount()<<" nodes with objective "
	   <<model.getObjValue()
	   <<(!model.status() ? " Finished" : " Not finished")
	   <<std::endl;

  // Print solution if finished - we can't get names from Osi!

  if (model.getMinimizationObjValue()<1.0e50) {
    // post process
    OsiSolverInterface * solver;
    if (preProcess) {
      process.postProcess(*model.solver());
      // Solution now back in solver1
      solver = & solver1;
    } else {
      solver = model.solver();
    }
    int numberColumns = solver->getNumCols();
    
    const double * solution = solver->getColSolution();
    
    int iColumn;
    std::cout<<std::setiosflags(std::ios::fixed|std::ios::showpoint)<<std::setw(14);

    std::cout<<"--------------------------------------"<<std::endl;
    for (iColumn=0;iColumn<numberColumns;iColumn++) {
      double value=solution[iColumn];
      if (fabs(value)>1.0e-7&&solver->isInteger(iColumn)) 
	std::cout<<std::setw(6)<<iColumn<<" "<<value<<std::endl;
    }
    std::cout<<"--------------------------------------"<<std::endl;
  
    std::cout<<std::resetiosflags(std::ios::fixed|std::ios::showpoint|std::ios::scientific);
  }
  return 0;
}

OsiClpSolverInterface solver_from_file(string str){
  OsiClpSolverInterface solver1;
  //string mpsFileName = "modele.lp";
  string mpsFileName = str;
  int numMpsReadErrors = solver1.readMps(mpsFileName.c_str(),"");
  assert(numMpsReadErrors==0);

  return solver1;
}

int test(){
  // model is as exmip1.mps from Data/samples
  int numberRows=5;
  int numberColumns=8;
  int numberElements=14;
  // matrix data - column ordered
  CoinBigIndex start[9]={0,2,4,6,8,10,11,12,14};
  int length[8]={2,2,2,2,2,1,1,2};
  int rows[14]={0,4,0,1,1,2,0,3,0,4,2,3,0,4};
  double elements[14]={3,5.6,1,2,1.1,1,-2,2.8,-1,1,1,-1.2,-1,1.9};
  CoinPackedMatrix matrix(true,numberRows,numberColumns,numberElements,elements,rows,start,length);
    
  // rim data
  double objective[8]={1,0,0,0,2,0,0,-1};
  double rowLower[5]={2.5,-COIN_DBL_MAX,4,1.8,3};
  double rowUpper[5]={COIN_DBL_MAX,2.1,4,5,15};
  double colLower[8]={2.5,0,0,0,0.5,0,0,0};
  double colUpper[8]={COIN_DBL_MAX,4.1,1,1,4,COIN_DBL_MAX,COIN_DBL_MAX,4.3};
  OsiClpSolverInterface solver1;
  // load problem
  solver1.loadProblem(matrix,colLower,colUpper,objective,
			 rowLower,rowUpper);
  // mark integer 
  solver1.setInteger(2);
  solver1.setInteger(3);

  return solve_model(solver1, true, false, 5, -1);

}

CbcModel makeCbcModel(OsiClpSolverInterface s){
    CbcModel c(s);
    return c;
}

void initialSolve(CbcModel& c){
    c.initialSolve();
}
void branchAndBound(CbcModel& c, int doStatistics){
    c.branchAndBound(doStatistics);
}
bool isProvenOptimal(CbcModel c){
    return c.isProvenOptimal();
}
bool isProvenInfeasible(CbcModel c){
    return c.isProvenInfeasible();
}
bool isContinuousUnbounded(CbcModel c){
    return c.isContinuousUnbounded();
}
bool isNodeLimitReached(CbcModel c){
    return c.isNodeLimitReached();
}
bool isSecondsLimitReached(CbcModel c){
    return c.isSecondsLimitReached();
}
int getNumCols(CbcModel c){
    return c.getNumCols();
}
int getNumRows(CbcModel c){
    return c.getNumRows();
}
CoinBigIndex getNumElements (CbcModel c){
    return c.getNumElements();
}
int numberIntegers(CbcModel c){
    return c.numberIntegers();
}
char integerType(CbcModel c,int i){
    return c.integerType(i);
}
const double* getColLower(CbcModel c){
    const double * col;
    col = new double[c.getNumCols()];
    col = c.getColLower();
}
const double* getColUpper(CbcModel c){
    const double * col;
    col = new double[c.getNumCols()];
    col = c.getColUpper();
}
const char* getRowSense(CbcModel c){
    return c.getRowSense();
}

const double* getRightHandSide(CbcModel c){
    return c.getRightHandSide();
}
const double* getRowRange(CbcModel c){
    return c.getRowRange();
}
const double* getRowLower(CbcModel c){
    return c.getRowLower();
}
const double* getRowUpper(CbcModel c){
    return c.getRowUpper();
}
const double* getObjCoefficients(CbcModel c){
    return c.getObjCoefficients();
}
double getObjSense(CbcModel c){
    return c.getObjSense();
}
void setObjSense (CbcModel & c, double s){
    c.setObjSense(s);
}

bool isContinuous(CbcModel c, int colIndex){
return c.isContinuous(colIndex);
}
bool isBinary(CbcModel c, int colIndex){
return c.isBinary(colIndex);
}
bool isInteger(CbcModel c, int colIndex){
    return c.isInteger(colIndex);
}
bool isIntegerNonBinary(CbcModel c, int colIndex){
    return c.isIntegerNonBinary(colIndex);
}
bool isFreeBinary(CbcModel c, int colIndex){
    return c.isFreeBinary(colIndex);
}
const CoinPackedMatrix* getMatrixByRow(CbcModel c){
    return c.getMatrixByRow();
}
const CoinPackedMatrix* getMatrixByCol(CbcModel c){
    return c.getMatrixByCol();
}
double getInfinity(CbcModel c){
    return c.getInfinity();
}
const double* getCbcColLower(CbcModel c){
    return c.getCbcColLower();
}
const double* getCbcColUpper(CbcModel c){
    return c.getCbcColUpper();
}
const double* getCbcRowLower(CbcModel c){
    return c.getCbcRowLower();
}
const double* getCbcRowUpper(CbcModel c){
    return c.getCbcRowUpper();
}
const double* getCbcColSolution(CbcModel c){
    return c.getCbcColSolution();
}
const double* getCbcRowPrice(CbcModel c){
    return c.getCbcRowPrice();
}
double getObjValue( CbcModel c){
    return c.getObjValue();
}
double* bestSolution(CbcModel c){
    return c.bestSolution();
}
int getNumberThreads(CbcModel c){
    return c.getNumberThreads();
}
void setNumberThreads(CbcModel& c, int value){
    c.setNumberThreads(value);
}

void setLogLevel(CbcModel& c, int n){
    c.setLogLevel(n);
}

double getCurrentObjValue(CbcModel c){
    return c.getCurrentObjValue();
}




//OsiClpInterface proxy methods
//Methods returning info on how the solution process terminated
bool isAbandoned (OsiClpSolverInterface osi){ //Are there a numerical difficulties?
    return osi.isAbandoned();
}
bool isProvenOptimal (OsiClpSolverInterface osi){ //Is optimality proven?
    return osi.isProvenOptimal();
}
bool isProvenPrimalInfeasible (OsiClpSolverInterface osi){ // Is primal infeasiblity proven?
    return osi.isProvenPrimalInfeasible();
}
bool isProvenDualInfeasible (OsiClpSolverInterface osi){ // Is dual infeasiblity proven?
    return osi.isProvenDualInfeasible();
 }
bool isPrimalObjectiveLimitReached (OsiClpSolverInterface osi){ // Is the given primal objective limit reached?
    return osi.isPrimalObjectiveLimitReached();
 }
bool isDualObjectiveLimitReached (OsiClpSolverInterface osi){ // Is the given dual objective limit reached?
    return osi.isDualObjectiveLimitReached();
}
bool isIterationLimitReached (OsiClpSolverInterface osi){ // Iteration limit reached?
    return osi.isIterationLimitReached();
}

//Methods related to querying the input data
int getNumCols (OsiClpSolverInterface osi){ // Get number of columns.
    return osi.getNumCols();
}
int getNumRows (OsiClpSolverInterface osi){ // Get number of rows.
    return osi.getNumRows();
}
int getNumElements (OsiClpSolverInterface osi){ // Get number of nonzero elements.
    return osi.getNumElements();
}
std::string getRowName (OsiClpSolverInterface osi, int rowIndex, unsigned maxLen){ // Return name of row if one exists or Rnnnnnnn maxLen is currently ignored and only there to match the signature from the base class!
    return osi.getRowName(rowIndex,maxLen);
}
std::string getColName (OsiClpSolverInterface osi, int colIndex, unsigned maxLen){ // Return name of column if one exists or Cnnnnnnn maxLen is currently ignored and only there to match the signature from the base class!
    return osi.getColName(colIndex,maxLen);
}
const double * getColLower (OsiClpSolverInterface osi){ // Get pointer to array[getNumCols(OsiClpSolverInterface osi)] of column lower bounds.
    return osi.getColLower();
}
const double * getColUpper (OsiClpSolverInterface osi){ // Get pointer to array[getNumCols(OsiClpSolverInterface osi)] of column upper bounds.
    return osi.getColUpper();
}
const char * getRowSense (OsiClpSolverInterface osi){ // Get pointer to array[getNumRows(OsiClpSolverInterface osi)] of row constraint senses.
    return osi.getRowSense();
}
const double * getRightHandSide (OsiClpSolverInterface osi){ // Get pointer to array[getNumRows(OsiClpSolverInterface osi)] of rows right-hand sides.
    return osi.getRightHandSide();
}
const double * getRowRange (OsiClpSolverInterface osi){ // Get pointer to array[getNumRows(OsiClpSolverInterface osi)] of row ranges.
    return osi.getRowRange();
}
const double * getRowLower (OsiClpSolverInterface osi){ // Get pointer to array[getNumRows(OsiClpSolverInterface osi)] of row lower bounds.
    return osi.getRowLower();
}
const double * getRowUpper (OsiClpSolverInterface osi){ // Get pointer to array[getNumRows(OsiClpSolverInterface osi)] of row upper bounds.
    return osi.getRowUpper();
}
const double * getObjCoefficients (OsiClpSolverInterface osi){ // Get pointer to array[getNumCols(OsiClpSolverInterface osi)] of objective function coefficients.
    return osi.getObjCoefficients();
}
double getObjSense (OsiClpSolverInterface osi){ // Get objective function sense (1 for min (default), -1 for max).
    return osi.getObjSense();
}
bool isContinuous (OsiClpSolverInterface osi, int colNumber){ // Return true if column is continuous.
    return osi.isContinuous(colNumber);
}
bool isBinary (OsiClpSolverInterface osi, int colIndex){ // Return true if variable is binary.
    return osi.isBinary(colIndex);
}
bool isInteger (OsiClpSolverInterface osi, int colIndex){ // Return true if column is integer.
    return osi.isInteger(colIndex);
}
bool isIntegerNonBinary (OsiClpSolverInterface osi, int colIndex){ // Return true if variable is general integer.
    return osi.isIntegerNonBinary(colIndex);
}
bool isFreeBinary (OsiClpSolverInterface osi, int colIndex){ // Return true if variable is binary and not fixed at either bound.
    return osi.isFreeBinary(colIndex);
}
const char * getColType (OsiClpSolverInterface osi, bool refresh){ // Return array of column length 0 - continuous 1 - binary (may get fixed later) 2 - general integer (may get fixed later).
    return osi.getColType(refresh);
}
const CoinPackedMatrix * getMatrixByRow (OsiClpSolverInterface osi){ // Get pointer to row-wise copy of matrix.
    return osi.getMatrixByRow();
}
const CoinPackedMatrix * getMatrixByCol (OsiClpSolverInterface osi){ // Get pointer to column-wise copy of matrix.
    return osi.getMatrixByCol();
}
CoinPackedMatrix *getMutableMatrixByCol (OsiClpSolverInterface osi){ // Get pointer to mutable column-wise copy of matrix.
    return osi.getMutableMatrixByCol();
}
double getInfinity (OsiClpSolverInterface osi){ // Get solver's value for infinity.
    return osi.getInfinity();
}



//Methods related to querying the solution
const double * 	getColSolution (OsiClpSolverInterface osi){ // Get pointer to array[getNumCols()] of primal solution vector.
    return osi.getColSolution();
}
const double * 	getRowPrice (OsiClpSolverInterface osi){ // Get pointer to array[getNumRows()] of dual prices.
    return osi.getRowPrice();
}
const double * 	getReducedCost (OsiClpSolverInterface osi){ // Get a pointer to array[getNumCols()] of reduced costs.
    return osi.getReducedCost();
}
const double * 	getRowActivity (OsiClpSolverInterface osi){ // Get pointer to array[getNumRows()] of row activity levels (constraint matrix times the solution vector.
    return osi.getRowActivity();
}
double getObjValue (OsiClpSolverInterface osi){ // Get objective function value.
    return osi.getObjValue();
}
int getIterationCount (OsiClpSolverInterface osi){ // Get how many iterations it took to solve the problem (whatever "iteration" mean to the solver.
    return osi.getIterationCount();
}
std::vector< double * > getDualRays (OsiClpSolverInterface osi, int maxNumRays){ // Get as many dual rays as the solver can provide.
    return osi.getDualRays(maxNumRays);
}
std::vector< double * > getPrimalRays (OsiClpSolverInterface osi, int maxNumRays){ // Get as many primal rays as the solver can provide.
    return osi.getPrimalRays(maxNumRays);
}



//Changing bounds on variables and constraints
void setObjCoeff (OsiClpSolverInterface & osi, int elementIndex, double elementValue){// Set an objective function coefficient.
    osi.setObjCoeff(elementIndex,elementValue);
}
void setColLower (OsiClpSolverInterface & osi, int elementIndex, double elementValue){// Set a single column lower bound, Use -DBL_MAX for -infinity.
    osi.setColLower(elementIndex,elementValue);
}
void setColUpper (OsiClpSolverInterface & osi, int elementIndex, double elementValue){// Set a single column upper bound, Use DBL_MAX for infinity.
    osi.setColUpper(elementIndex,elementValue);
}
void setColBounds (OsiClpSolverInterface & osi, int elementIndex, double lower, double upper){// Set a single column lower and upper bound.
    osi.setColBounds(elementIndex,lower,upper);
}
void setColSetBounds (OsiClpSolverInterface & osi, const int *indexFirst, const int *indexLast, const double *boundList){// Set the bounds on a number of columns simultaneously, The default implementation just invokes setColLower() and setColUpper() over and over again.
    osi.setColSetBounds(indexFirst,indexLast,boundList);
}
void setRowLower (OsiClpSolverInterface & osi, int elementIndex, double elementValue){// Set a single row lower bound, Use -DBL_MAX for -infinity.
    osi.setRowLower(elementIndex,elementValue);
}
void setRowUpper (OsiClpSolverInterface & osi, int elementIndex, double elementValue){// Set a single row upper bound, Use DBL_MAX for infinity.
    osi.setRowUpper(elementIndex,elementValue);
}
void setRowBounds (OsiClpSolverInterface & osi, int elementIndex, double lower, double upper){// Set a single row lower and upper bound.
    osi.setRowBounds(elementIndex,lower,upper);
}
void setRowType (OsiClpSolverInterface & osi, int index, char sense, double rightHandSide, double range){// Set the type of a single row.
    osi.setRowType(index,sense,rightHandSide,range);
}
void setRowSetBounds (OsiClpSolverInterface & osi, const int *indexFirst, const int *indexLast, const double *boundList){// Set the bounds on a number of rows simultaneously. The default implementation just invokes setRowLower() and setRowUpper() over and over again.
    osi.setRowSetBounds(indexFirst,indexLast,boundList);
}
void setRowSetTypes (OsiClpSolverInterface & osi, const int *indexFirst, const int *indexLast, const char *senseList, const double *rhsList, const double *rangeList){// Set the type of a number of rows simultaneously. The default implementation just invokes setRowType() over and over again.
    osi.setRowSetTypes(indexFirst,indexLast,senseList,rhsList,rangeList);
}
void setObjective (OsiClpSolverInterface & osi, const double *array){// Set the objective coefficients for all columns array [getNumCols()] is an array of values for the objective.
    osi.setObjective(array);
}
void setColLower (OsiClpSolverInterface & osi, const double *array){// Set the lower bounds for all columns array [getNumCols()] is an array of values for the objective.
    osi.setColLower(array);
}
void setColUpper (OsiClpSolverInterface & osi, const double *array){// Set the upper bounds for all columns array [getNumCols()] is an array of values for the objective.
    osi.setColUpper(array);
}
void setRowName (OsiClpSolverInterface & osi, int rowIndex, std::string name){// Set name of row.
    osi.setRowName(rowIndex,name);
}
void setColName (OsiClpSolverInterface & osi, int colIndex, std::string name){// Set name of column.
    osi.setColName(colIndex,name);
}


//Integrality related changing methods
void setContinuous (OsiClpSolverInterface & osi, int index){// Set the index-th variable to be a continuous variable.
    osi.setContinuous(index);
}
void setInteger (OsiClpSolverInterface & osi, int index){// Set the index-th variable to be an integer variable.
    osi.setInteger(index);
}
void setContinuous (OsiClpSolverInterface & osi, const int *indices, int len){// Set the variables listed in indices (which is of length len) to be continuous variables.
    osi.setContinuous(indices,len);
}
void setInteger (OsiClpSolverInterface & osi, const int *indices, int len){// Set the variables listed in indices (which is of length len) to be integer variables.
    osi.setInteger(indices,len);
}



/*Methods to expand a problem.
  Note that if a column is added then by default
  it will correspond to a continuous variable.  */

void addCol (OsiClpSolverInterface & osi, const CoinPackedVectorBase &vec, const double collb, const double colub, const double obj){// Add a column (primal variable) to the problem.
    osi.addCol(vec,collb,colub,obj);
}
void addCol (OsiClpSolverInterface & osi, int numberElements, const int *rows, const double *elements, const double collb, const double colub, const double obj){// Add a column (primal variable) to the problem.
    osi.addCol(numberElements,rows,elements,collb,colub,obj);
}
void addCols (OsiClpSolverInterface & osi, const int numcols, const CoinPackedVectorBase *const *cols, const double *collb, const double *colub, const double *obj){// Add a set of columns (primal variables) to the problem.
    osi.addCols (numcols,cols,collb,colub,obj);
}
void addCols (OsiClpSolverInterface & osi, const int numcols, const int *columnStarts, const int *rows, const double *elements, const double *collb, const double *colub, const double *obj){// Add a set of columns (primal variables) to the problem.
    osi.addCols (numcols,columnStarts,rows,elements,collb,colub,obj);
}
void deleteCols (OsiClpSolverInterface & osi, const int num, const int *colIndices){// Remove a set of columns (primal variables) from the problem.
    osi.deleteCols(num,colIndices);
}
void addRow (OsiClpSolverInterface & osi, const CoinPackedVectorBase &vec, const double rowlb, const double rowub){// Add a row (constraint) to the problem.
    osi.addRow(vec,rowlb,rowub);
}
void addRow (OsiClpSolverInterface & osi, const CoinPackedVectorBase &vec, const char rowsen, const double rowrhs, const double rowrng){// Add a row (constraint) to the problem.
    osi.addRow(vec,rowsen,rowrhs,rowrng);
}
void addRow (OsiClpSolverInterface & osi, int numberElements, const int *columns, const double *element, const double rowlb, const double rowub){// Add a row (constraint) to the problem.
    osi.addRow(numberElements,columns,element,rowlb,rowub);
}
void addRows (OsiClpSolverInterface & osi, const int numrows, const CoinPackedVectorBase *const *rows, const double *rowlb, const double *rowub){
    osi.addRows(numrows,rows,rowlb,rowub);
}
void addRows (OsiClpSolverInterface & osi, const int numrows, const CoinPackedVectorBase *const *rows, const char *rowsen, const double *rowrhs, const double *rowrng){// Add a set of rows (constraints) to the problem.
    osi.addRows(numrows,rows,rowsen,rowrhs,rowrng);
}
void addRows (OsiClpSolverInterface & osi, const int numrows, const int *rowStarts, const int *columns, const double *element, const double *rowlb, const double *rowub){// Add a set of rows (constraints) to the problem.
    osi.addRows(numrows,rowStarts,columns,element,rowlb,rowub);
}
void modifyCoefficient (OsiClpSolverInterface & osi, int row, int column, double newElement, bool keepZero){
    osi.modifyCoefficient(row,column,newElement,keepZero);
}
void deleteRows (OsiClpSolverInterface & osi, const int num, const int *rowIndices){// Delete a set of rows (constraints) from the problem.
    osi.deleteRows(num,rowIndices);
}
void saveBaseModel (OsiClpSolverInterface & osi){// If solver wants it can save a copy of "base" (continuous) model here.
    osi.saveBaseModel();
}
void restoreBaseModel (OsiClpSolverInterface & osi, int numberRows){// Strip off rows to get to this number of rows.
    osi.restoreBaseModel(numberRows);
}
void applyRowCuts (OsiClpSolverInterface & osi, int numberCuts, const OsiRowCut *cuts){// Apply a collection of row cuts which are all effective.
    osi.applyRowCuts(numberCuts,cuts);
}
void applyRowCuts (OsiClpSolverInterface & osi, int numberCuts, const OsiRowCut **cuts){// Apply a collection of row cuts which are all effective.
    osi.applyRowCuts(numberCuts,cuts);
}



//Methods to input a problem
void loadProblem (OsiClpSolverInterface & osi, const CoinPackedMatrix &matrix, const double *collb, const double *colub, const double *obj, const double *rowlb, const double *rowub){// Load in an problem by copying the arguments (the constraints on the rows are given by lower and upper bounds).
    osi.loadProblem(matrix,collb,colub,obj,rowlb,rowub);
}
void assignProblem (OsiClpSolverInterface & osi, CoinPackedMatrix *&matrix, double *&collb, double *&colub, double *&obj, double *&rowlb, double *&rowub){// Load in an problem by assuming ownership of the arguments (the constraints on the rows are given by lower and upper bounds).
    osi.assignProblem(matrix,collb,colub,obj,rowlb,rowub);
}
void loadProblem (OsiClpSolverInterface & osi, const CoinPackedMatrix &matrix, const double *collb, const double *colub, const double *obj, const char *rowsen, const double *rowrhs, const double *rowrng){// Load in an problem by copying the arguments (the constraints on the rows are given by sense/rhs/range triplets).
    osi.loadProblem(matrix,collb,colub,obj,rowsen,rowrhs,rowrng);
}
void assignProblem (OsiClpSolverInterface & osi, CoinPackedMatrix *&matrix, double *&collb, double *&colub, double *&obj, char *&rowsen, double *&rowrhs, double *&rowrng){// Load in an problem by assuming ownership of the arguments (the constraints on the rows are given by sense/rhs/range triplets).
    osi.assignProblem(matrix,collb,colub,obj,rowsen,rowrhs,rowrng);
}
void loadProblem (OsiClpSolverInterface & osi, const int numcols, const int numrows, const CoinBigIndex *start, const int *index, const double *value, const double *collb, const double *colub, const double *obj, const double *rowlb, const double *rowub){// Just like the other loadProblem() methods except that the matrix is given in a standard column major ordered format (without gaps).
    osi.loadProblem(numcols,numrows,start,index,value,collb,colub,obj,rowlb,rowub);
}
void loadProblem (OsiClpSolverInterface & osi, const int numcols, const int numrows, const CoinBigIndex *start, const int *index, const double *value, const double *collb, const double *colub, const double *obj, const char *rowsen, const double *rowrhs, const double *rowrng){// Just like the other loadProblem() methods except that the matrix is given in a standard column major ordered format (without gaps).
    osi.loadProblem(numcols,numrows,start,index,value,collb,colub,obj,rowsen,rowrhs,rowrng);
}
int loadFromCoinModel (OsiClpSolverInterface & osi, CoinModel &modelObject, bool keepSolution){// This loads a model from a coinModel object - returns number of errors.
    osi.loadFromCoinModel(modelObject,keepSolution);
}
int readMps (OsiClpSolverInterface & osi, const char *filename, const char *extension){// Read an mps file from the given filename (defaults to Osi reader) - returns number of errors (see OsiMpsReader class).
    osi.readMps(filename,extension);
}
int readMps (OsiClpSolverInterface & osi, const char *filename, bool keepNames, bool allowErrors){// Read an mps file from the given filename returns number of errors (see OsiMpsReader class).
    osi.readMps(filename,keepNames,allowErrors);
}
void writeMps (OsiClpSolverInterface & osi, const char *filename, const char *extension, double objSense){// Write the problem into an mps file of the given filename.
    osi.writeMps(filename,extension,objSense);
}
int writeMpsNative (OsiClpSolverInterface & osi, const char *filename, const char **rowNames, const char **columnNames, int formatType, int numberAcross, double objSense){// Write the problem into an mps file of the given filename, names may be null.
    osi.writeMpsNative(filename,rowNames,columnNames,formatType,numberAcross,objSense);
}
int readLp (OsiClpSolverInterface & osi, const char *filename, const double epsilon){// Read file in LP format (with names).
    osi.readLp(filename,epsilon);
}
void writeLp (OsiClpSolverInterface & osi, const char *filename, const char *extension, double epsilon, int numberAcross, int decimals, double objSense, bool useRowNames){// Write the problem into an Lp file of the given filename.
    osi.writeLp(filename,extension,epsilon,numberAcross,decimals,objSense,useRowNames);
}
void writeLp (OsiClpSolverInterface & osi, FILE *fp, double epsilon, int numberAcross, int decimals, double objSense, bool useRowNames){// Write the problem into the file pointed to by the parameter fp.
    osi.writeLp(fp,epsilon,numberAcross,decimals,objSense,useRowNames);
}


